from orca import plugin

import gi
gi.require_version('Peas', '1.0')
from gi.repository import GObject
from gi.repository import Peas

import ObjectNavigator

def _(text):
    # for compatibility til we have translation
    return text

class ObjectNavigation(GObject.Object, Peas.Activatable, plugin.Plugin):
    __gtype_name__ = 'ObjectNavigation'

    object = GObject.Property(type=GObject.Object)
    def __init__(self):
        plugin.Plugin.__init__(self)
        self.navigator = None
        self.keybindings = {}

    def do_activate(self):
        API = self.object
        self.navigator = ObjectNavigator.ObjectNavigator(API)
        # bind keys
        self.keybindings['navigatorUpHandler'] = API.app.getAPIHelper().registerShortcut(self.navigatorUp, "Up", _("Moves the object navigator to the parent of the object with navigator focus."), ctrlKey = True)
        self.keybindings['navigatorDownHandler'] = API.app.getAPIHelper().registerShortcut(self.navigatorDown, "Down", _("Moves the object navigator to the first child of the object with navigator focus."), ctrlKey = True)
        self.keybindings['navigatorNextHandler'] = API.app.getAPIHelper().registerShortcut(self.navigatorNext, "Right", _("Moves the object navigator to the next sibling of the object with navigator focus."), ctrlKey = True)
        self.keybindings['navigatorPreviousHandler'] = API.app.getAPIHelper().registerShortcut(self.navigatorPrevious, "Left", _("Moves the object navigator to the previous sibling of the object with navigator focus."), ctrlKey = True)
        self.keybindings['navigatorActionHandler'] = API.app.getAPIHelper().registerShortcut(self.navigatorAction, "Return", _("Performs the usual action on the object with navigator focus."), ctrlKey = True)
        self.keybindings['navigatorToggleSimplifyHandler'] = API.app.getAPIHelper().registerShortcut(self.navigatorToggleSimplify, "S", _("Toggles whether object navigation should be simplified."), ctrlKey = True)
        print('ObjectNavigation activate')

    def do_deactivate(self):
        API = self.object
        for function, keybinding in self.keybindings.items():
            API.app.getAPIHelper().unregisterShortcut(keybinding)
        self.keybindings = {}
        self.navigator = None
        print('ObjectNavigation deactivate')
    def do_update_state(self):
        API = self.object

    def navigatorUp(self, script, inputEvent):
        """ Moves the object navigator to the parent of the current object. """
        self.navigator.up()
        return True 

    def navigatorDown(self, script, inputEvent):
        """ Moves the object navigator to the first child of the current object. """
        self.navigator.down()
        return True

    def navigatorNext(self, script, inputEvent):
        """ Moves the object navigator to the next object. """
        self.navigator.next()
        return True

    def navigatorPrevious(self, script, inputEvent):
        """ Moves the object navigator to the previous object. """
        self.navigator.previous()
        return True

    def navigatorAction(self, script, inputEvent):
        """ Performs an action (a click, press or focus) on the object with navigator focus. """
        API = self.object
        obj = self.navigator.navigatorFocus 
        eventsynthesizer = API.app.getDynamicApiManager().getAPI('EventSynthesizer')

        if not obj:
            return False
        try:
            if eventsynthesizer.clickActionOn(obj):
                return True
        except:
            pass
        try:
            if eventsynthesizer.pressActionOn(obj):
                return True
        except:
            pass
        try:
            if eventsynthesizer.grabFocusOn(obj):
                return True
        except:
            pass
        return False

    def navigatorToggleSimplify(self, script, inputEvent):
        """ Handles toggling simplified navigation with the object navigator. """
        API = self.object
        self.navigator.simplify = not self.navigator.simplify
        message = ''
        if self.navigator.simplify:
            message = "Simplified navigation enabled."
        else:
            message = "Simplified navigation DISABLED."
        API.app.getDynamicApiManager().getAPI('OrcaState').activeScript.presentMessage(message, resetStyles=False)

        return True
